@@ -92,7 +92,7 @@ public class PdfBoxGraphics2D extends Graphics2D
     private Shape clipShape;
     private Color backgroundColor;
     private final CopyInfo copyInfo;
-    private final PDRectangle bbox;
+    final PDRectangle bbox;
 
     /**
      * Set a new color mapper.
@@ -258,6 +258,7 @@ public class PdfBoxGraphics2D extends Graphics2D
             this.fontTextDrawer = parentGfx.fontTextDrawer;
             this.imageEncoder = parentGfx.imageEncoder;
             this.paintApplier = parentGfx.paintApplier;
+            this.drawControl = parentGfx.drawControl;
         }
 
         baseTransform = new AffineTransform();
@@ -271,26 +272,26 @@ public class PdfBoxGraphics2D extends Graphics2D
 
     }
 
-	/**
-	 * Sometimes you need to access the PDResources and add special resources to it
-	 * for some stuff (e.g. patterns of embedded PDFs or simmilar). For that you
-	 * need the PDResources associated with the XForm.
-	 *
-	 * It's identlical with getXFormObject().getResources(), with the difference
-	 * beeing that you can access it while the Graphics2D is not yet disposed.
-	 *
-	 * @return the PDResources of the resulting XForm
-	 */
+    /**
+     * Sometimes you need to access the PDResources and add special resources to it
+     * for some stuff (e.g. patterns of embedded PDFs or simmilar). For that you
+     * need the PDResources associated with the XForm.
+     * <p>
+     * It's identlical with getXFormObject().getResources(), with the difference
+     * beeing that you can access it while the Graphics2D is not yet disposed.
+     *
+     * @return the PDResources of the resulting XForm
+     */
     public PDResources getResources()
-	{
+    {
         return xFormObject.getResources();
     }
 
-	/**
-	 * *AFTER* you have disposed() this Graphics2D you can access the XForm
-	 *
-	 * @return the PDFormXObject which resulted in this graphics
-	 */
+    /**
+     * *AFTER* you have disposed() this Graphics2D you can access the XForm
+     *
+     * @return the PDFormXObject which resulted in this graphics
+     */
     @SuppressWarnings("WeakerAccess")
     public PDFormXObject getXFormObject()
     {
@@ -542,7 +543,7 @@ public class PdfBoxGraphics2D extends Graphics2D
                     dashArray[i] = calculateTransformedLength(dashArray[i], tf);
 
                 contentStream.setLineDashPattern(dashArray,
-                    calculateTransformedLength(basicStroke.getDashPhase(), tf));
+                        calculateTransformedLength(basicStroke.getDashPhase(), tf));
             }
         }
         else if (strokeToApply != null)
@@ -552,7 +553,8 @@ public class PdfBoxGraphics2D extends Graphics2D
         }
     }
 
-    private float calculateTransformedLength(float length, AffineTransform tf) {
+    private float calculateTransformedLength(float length, AffineTransform tf)
+    {
         // Represent stroke width as a horizontal line from origin to basicStroke.LineWidth.
         Point2D.Float lengthVector = new Point2D.Float(length, 0);
         // Apply the current transform to the horizontal line.
@@ -650,9 +652,21 @@ public class PdfBoxGraphics2D extends Graphics2D
     public boolean drawImage(Image img, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1,
             int sx2, int sy2, ImageObserver observer)
     {
-        return drawImage(img, dx1, dy1, dx2, dy2, sx1, sy2, sx2, sy2, null, observer);
+        return drawImage(img, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null, observer);
     }
 
+    final IPdfBoxGraphics2DImageEncoder.IPdfBoxGraphics2DImageEncoderEnv imageEncoderEnv = new IPdfBoxGraphics2DImageEncoder.IPdfBoxGraphics2DImageEncoderEnv()
+    {
+        @Override
+        public ImageInterpolation getImageInterpolation()
+        {
+            Object renderingHint = getRenderingHint(RenderingHints.KEY_INTERPOLATION);
+            if (renderingHint == RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR)
+                return ImageInterpolation.NearestNeigbor;
+            return ImageInterpolation.Interpolate;
+        }
+    };
+
     public boolean drawImage(Image img, AffineTransform xform, ImageObserver obs)
     {
         checkNoCopyActive();
@@ -662,7 +676,8 @@ public class PdfBoxGraphics2D extends Graphics2D
         if (xform != null)
             tf.concatenate((AffineTransform) xform.clone());
 
-        PDImageXObject pdImage = imageEncoder.encodeImage(document, contentStream, img);
+        PDImageXObject pdImage = imageEncoder.encodeImage(document, contentStream, img,
+                imageEncoderEnv);
         try
         {
             contentStreamSaveState();
@@ -671,10 +686,6 @@ public class PdfBoxGraphics2D extends Graphics2D
             tf.scale(1, -1);
             contentStream.transform(new Matrix(tf));
 
-            Object keyInterpolation = renderingHints.get(RenderingHints.KEY_INTERPOLATION);
-            if (RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR.equals(keyInterpolation))
-                pdImage.setInterpolate(false);
-
             if (composite != null)
             {
                 // We got an AlphaComposite, we must set the extended graphics dictionary correctly.
@@ -713,7 +724,7 @@ public class PdfBoxGraphics2D extends Graphics2D
              */
             if (bgcolor != null)
             {
-                contentStream.setNonStrokingColor(colorMapper.mapColor( bgcolor, colorMapperEnv));
+                contentStream.setNonStrokingColor(colorMapper.mapColor(bgcolor, colorMapperEnv));
                 walkShape(new Rectangle(dx1, dy1, width, height));
                 contentStream.fill();
             }
@@ -1021,14 +1032,17 @@ public class PdfBoxGraphics2D extends Graphics2D
     }
 
     private final PaintEnvImpl paintEnv = new PaintEnvImpl();
-    final IColorMapperEnv colorMapperEnv = new IColorMapperEnv() {
+    final IColorMapperEnv colorMapperEnv = new IColorMapperEnv()
+    {
         @Override
-        public PDPageContentStream getContentStream() {
+        public PDPageContentStream getContentStream()
+        {
             return contentStream;
         }
 
         @Override
-        public PDResources getResources() {
+        public PDResources getResources()
+        {
             return PdfBoxGraphics2D.this.getResources();
         }
     };
@@ -1476,6 +1490,7 @@ public class PdfBoxGraphics2D extends Graphics2D
         throw new RuntimeException(e);
     }
 
+    @Override
     public void copyArea(int x, int y, int width, int height, int dx, int dy)
     {
         /*
@@ -1484,16 +1499,24 @@ public class PdfBoxGraphics2D extends Graphics2D
         throw new IllegalStateException("copyArea() not possible!");
     }
 
+    @Override
     public void drawLine(int x1, int y1, int x2, int y2)
     {
         draw(new Line2D.Double(x1, y1, x2, y2));
     }
 
+    @Override
     public void fillRect(int x, int y, int width, int height)
     {
         fill(new Rectangle(x, y, width, height));
     }
 
+    @Override
+    public void drawRect(int x, int y, int width, int height)
+    {
+        draw(new Rectangle(x, y, width, height));
+    }
+
     public void clearRect(int x, int y, int width, int height)
     {
         Paint p = paint;
