@@ -1,6 +1,13 @@
 package de.rototor.pdfbox.graphics2d;
 
-import org.apache.pdfbox.cos.*;
+import org.apache.pdfbox.cos.COSArray;
+import org.apache.pdfbox.cos.COSBase;
+import org.apache.pdfbox.cos.COSBoolean;
+import org.apache.pdfbox.cos.COSDictionary;
+import org.apache.pdfbox.cos.COSFloat;
+import org.apache.pdfbox.cos.COSInteger;
+import org.apache.pdfbox.cos.COSName;
+import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.multipdf.PDFCloneUtility;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDPageContentStream;
@@ -8,11 +15,16 @@ import org.apache.pdfbox.pdmodel.PDResources;
 import org.apache.pdfbox.pdmodel.common.COSObjectable;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.common.function.PDFunctionType3;
+import org.apache.pdfbox.pdmodel.graphics.PDXObject;
 import org.apache.pdfbox.pdmodel.graphics.color.PDColor;
 import org.apache.pdfbox.pdmodel.graphics.color.PDColorSpace;
+import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceGray;
 import org.apache.pdfbox.pdmodel.graphics.color.PDPattern;
 import org.apache.pdfbox.pdmodel.graphics.form.PDFormXObject;
+import org.apache.pdfbox.pdmodel.graphics.form.PDTransparencyGroup;
+import org.apache.pdfbox.pdmodel.graphics.image.LosslessFactory;
 import org.apache.pdfbox.pdmodel.graphics.image.PDImageXObject;
+import org.apache.pdfbox.pdmodel.graphics.pattern.PDShadingPattern;
 import org.apache.pdfbox.pdmodel.graphics.pattern.PDTilingPattern;
 import org.apache.pdfbox.pdmodel.graphics.shading.PDShading;
 import org.apache.pdfbox.pdmodel.graphics.shading.PDShadingType3;
@@ -28,8 +40,11 @@ import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.io.IOException;
 import java.lang.reflect.Method;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
 import java.util.List;
-import java.util.*;
+import java.util.Map;
 
 /**
  * Default paint mapper.
@@ -39,8 +54,29 @@ import java.util.*;
 public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintApplier
 {
 
+    interface ShadingMaskModifier
+    {
+        PDShading applyMasking(PaintApplierState state, PDShading pdShading) throws IOException;
+    }
+
+    static class IdentityShadingMaskModifier implements ShadingMaskModifier
+    {
+
+        @Override
+        public PDShading applyMasking(PaintApplierState state, PDShading pdShading)
+        {
+            return pdShading;
+        }
+
+        final static IdentityShadingMaskModifier INSTANCE = new IdentityShadingMaskModifier();
+    }
+
+    /**
+     * Internal State of the PaintApplyer. Allows derived classes to
+     * change some states.
+     */
     @SuppressWarnings("WeakerAccess")
-    protected static class PaintApplierState
+    public static class PaintApplierState
     {
         protected PDDocument document;
         protected PDPageContentStream contentStream;
@@ -57,13 +93,16 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         private COSDictionary dictExtendedState;
         private IPaintEnv env;
         private IPdfBoxGraphics2DColorMapper.IColorMapperEnv colorMapperEnv;
-        public AffineTransform tf;
+        private IPdfBoxGraphics2DImageEncoder.IPdfBoxGraphics2DImageEncoderEnv imageEncoderEnv;
+        AffineTransform tf;
         /**
          * This transform is only set, when we apply a nested
          * paint (e.g. a TilingPattern's paint)
          */
         protected AffineTransform nestedTransform;
 
+        private ShadingMaskModifier shadingMaskModifier = IdentityShadingMaskModifier.INSTANCE;
+
         private void ensureExtendedState()
         {
             if (pdExtendedGraphicsState == null)
@@ -74,6 +113,82 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
             }
             assert pdExtendedGraphicsState != null;
         }
+
+        /**
+         * Setup a mask for the next fill/stroke operation
+         *
+         * @param image       the mask image. It will be used to generate a grayscale image, that
+         *                    will directly map to the alpha channel.
+         * @param boundingBox The bounding box of the masking. I.e. where to apply the mask
+         */
+        public void setupLuminosityMasking(BufferedImage image, PDRectangle boundingBox)
+                throws IOException
+        {
+            PDImageXObject pdMask = LosslessFactory.createFromImage(document, image);
+            setupLuminosityMasking(pdMask, boundingBox);
+        }
+
+        /**
+         * Setup a mask for the next fill/stroke operation. It will use the BBox of the
+         * formXObject.
+         *
+         * @param formXObject the mask form. It will be used to generate a grayscale image, that
+         *                    will directly map to the alpha channel.
+         */
+        public void setupLuminosityMasking(PDFormXObject formXObject) throws IOException
+        {
+            setupLuminosityMasking(formXObject, formXObject.getBBox());
+        }
+
+        /**
+         * Setup a mask for the next fill/stroke operation.
+         *
+         * @param maskXObject a PDXObject (form or image) which generates a grayscale image for the masking.
+         *                    This image will directly map to the alpha channel.
+         * @param boundingBox The bounding box of the masking. I.e. where to apply the mask
+         */
+        public void setupLuminosityMasking(PDXObject maskXObject, PDRectangle boundingBox)
+                throws IOException
+        {
+            ensureExtendedState();
+            pdExtendedGraphicsState.setAlphaSourceFlag(false);
+            pdExtendedGraphicsState.setNonStrokingAlphaConstant(null);
+            pdExtendedGraphicsState.setStrokingAlphaConstant(null);
+
+            PDTransparencyGroup groupXForm = new PDTransparencyGroup(document);
+            groupXForm.setResources(new PDResources());
+            COSDictionary groupDict = new COSDictionary();
+            groupDict.setItem(COSName.S, COSName.TRANSPARENCY);
+            groupDict.setItem(COSName.CS, COSName.DEVICEGRAY);
+            groupDict.setItem(COSName.TYPE, COSName.GROUP);
+            groupXForm.getCOSObject().setItem(COSName.GROUP, groupDict);
+            groupXForm.setBBox(boundingBox);
+            groupXForm.setFormType(1);
+            resources.add(groupXForm);
+
+            PDPageContentStream grpContentStream = new PDPageContentStream(document, groupXForm,
+                    groupXForm.getStream().createOutputStream(COSName.FLATE_DECODE));
+            if (maskXObject instanceof PDFormXObject)
+            {
+                grpContentStream.drawForm((PDFormXObject) maskXObject);
+            }
+            else if (maskXObject instanceof PDImageXObject)
+            {
+                Matrix matrix = new Matrix();
+                PDImageXObject imageXObject = (PDImageXObject) maskXObject;
+                matrix.scale(boundingBox.getWidth(), boundingBox.getHeight());
+                grpContentStream.drawImage(imageXObject, matrix);
+            }
+
+            grpContentStream.close();
+
+            COSDictionary mask = new COSDictionary();
+            mask.setItem(COSName.G, groupXForm);
+            mask.setItem(COSName.S, COSName.LUMINOSITY);
+            mask.setItem(COSName.TYPE, COSName.MASK);
+
+            dictExtendedState.setItem(COSName.SMASK, mask);
+        }
     }
 
     private final ExtGStateCache extGStateCache = new ExtGStateCache();
@@ -92,7 +207,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         state.composite = env.getComposite();
         state.pdExtendedGraphicsState = null;
         state.env = env;
-        state.colorMapperEnv = env.getGraphics2D().colorMapperEnv;;
+        state.colorMapperEnv = env.getGraphics2D().colorMapperEnv;
+        state.imageEncoderEnv = env.getGraphics2D().imageEncoderEnv;
         state.tf = tf;
         state.nestedTransform = null;
         PDShading shading = applyPaint(paint, state);
@@ -107,8 +223,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
     {
         PDPageContentStream contentStream = state.contentStream;
         IPdfBoxGraphics2DColorMapper colorMapper = state.colorMapper;
-		contentStream.setStrokingColor(colorMapper.mapColor(color, state.colorMapperEnv));
-		contentStream.setNonStrokingColor(colorMapper.mapColor(color, state.colorMapperEnv));
+        contentStream.setStrokingColor(colorMapper.mapColor(color, state.colorMapperEnv));
+        contentStream.setNonStrokingColor(colorMapper.mapColor(color, state.colorMapperEnv));
 
         int alpha = color.getAlpha();
         if (alpha < 255)
@@ -120,34 +236,36 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
             Float strokingAlphaConstant = state.pdExtendedGraphicsState.getStrokingAlphaConstant();
             if (strokingAlphaConstant == null)
                 strokingAlphaConstant = 1f;
-            state.pdExtendedGraphicsState
-                    .setStrokingAlphaConstant(strokingAlphaConstant * (alpha / 255f));
-            Float nonStrokingAlphaConstant = state.pdExtendedGraphicsState
-                    .getNonStrokingAlphaConstant();
+            state.pdExtendedGraphicsState.setStrokingAlphaConstant(
+                    strokingAlphaConstant * (alpha / 255f));
+            Float nonStrokingAlphaConstant = state.pdExtendedGraphicsState.getNonStrokingAlphaConstant();
             if (nonStrokingAlphaConstant == null)
                 nonStrokingAlphaConstant = 1f;
-            state.pdExtendedGraphicsState
-                    .setNonStrokingAlphaConstant(nonStrokingAlphaConstant * (alpha / 255f));
+            state.pdExtendedGraphicsState.setNonStrokingAlphaConstant(
+                    nonStrokingAlphaConstant * (alpha / 255f));
         }
 
-		if (color instanceof IPdfBoxGraphics2DColor)
-		{
-			if (((IPdfBoxGraphics2DColor) color).isOverprint())
-			{
-				state.ensureExtendedState();
-				state.pdExtendedGraphicsState.setOverprintMode(1.0f);
+        if (color instanceof IPdfBoxGraphics2DColor)
+        {
+            if (((IPdfBoxGraphics2DColor) color).isOverprint())
+            {
+                state.ensureExtendedState();
+                state.pdExtendedGraphicsState.setOverprintMode(1.0f);
                 /*
                  * Till a fixed version of PDFBOX for PDFBOX-5361 is available,
-                 * we make do this workaround
+                 * we do this workaround
                  */
                 state.dictExtendedState.setItem(COSName.OPM, COSInteger.get(1));
-				state.pdExtendedGraphicsState.setNonStrokingOverprintControl(true);
-				state.pdExtendedGraphicsState.setStrokingOverprintControl(true);
+                state.pdExtendedGraphicsState.setNonStrokingOverprintControl(true);
+                state.pdExtendedGraphicsState.setStrokingOverprintControl(true);
             }
         }
     }
 
-    private PDShading applyPaint(Paint paint, PaintApplierState state) throws IOException
+    /**
+     * Apply paint and graphic state
+     */
+    protected PDShading applyPaint(Paint paint, PaintApplierState state) throws IOException
     {
         applyComposite(state);
 
@@ -189,12 +307,12 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         else if (paint instanceof ShadingPaint)
         {
             // PDFBox paint, we can import the shading directly
-            return shadingCache
-                    .makeUnqiue(importPDFBoxShadingPaint((ShadingPaint<?>) paint, state));
+            return shadingCache.makeUnqiue(
+                    importPDFBoxShadingPaint((ShadingPaint<?>) paint, state));
         }
         else
         {
-            System.err.printf("Don't know paint %s" , paint.getClass().getName());
+            System.err.printf("Don't know paint %s", paint.getClass().getName());
         }
 
         return null;
@@ -214,8 +332,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         PDShading shading = paint.getShading();
 
         state.contentStream.transform(matrix);
-        return PDShading.create((COSDictionary) pdfCloneUtility
-                .cloneForNewDocument(shading.getCOSObject()));
+        return PDShading.create(
+                (COSDictionary) pdfCloneUtility.cloneForNewDocument(shading.getCOSObject()));
     }
 
     /*
@@ -261,9 +379,9 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         }
         catch (Exception e)
         {
-            System.err
-                    .printf("PdfBoxGraphics2DPaintApplier error while drawing Batik PatternPaint %s",
-                            e.getMessage());
+            System.err.printf(
+                    "PdfBoxGraphics2DPaintApplier error while drawing Batik PatternPaint %s",
+                    e.getMessage());
             return;
         }
         pdfBoxGraphics2D.dispose();
@@ -296,7 +414,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         }
         catch (Exception e)
         {
-            System.err.printf("PdfBoxGraphics2DPaintApplier error while drawing Tiling Paint %s", e.getMessage());
+            System.err.printf("PdfBoxGraphics2DPaintApplier error while drawing Tiling Paint %s",
+                    e.getMessage());
         }
     }
 
@@ -350,6 +469,7 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
             blendMode = COSName.NORMAL;
             break;
         case AlphaComposite.SRC_OVER:
+            //noinspection ConstantConditions
             blendMode = COSName.COMPATIBLE;
             break;
         case AlphaComposite.XOR:
@@ -360,6 +480,7 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         case AlphaComposite.DST_ATOP:
             break;
         case AlphaComposite.SRC_ATOP:
+            //noinspection ConstantConditions
             blendMode = COSName.COMPATIBLE;
             break;
         case AlphaComposite.DST_IN:
@@ -386,6 +507,16 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
      */
     private static final double EPSILON = 0.00001;
 
+    static boolean haveColorsTransparency(Color[] colors)
+    {
+        for (Color c : colors)
+        {
+            if (c.getAlpha() != 255)
+                return true;
+        }
+        return false;
+    }
+
     private PDShading buildLinearGradientShading(Paint paint, PaintApplierState state)
             throws IOException
     {
@@ -415,14 +546,16 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
          */
         state.env.ensureShapeIsWalked();
 
+        final PDShading shading;
         if (isObjectBoundingBox)
         {
-            return linearGradientObjectBoundingBoxShading(paint, state);
+            shading = linearGradientObjectBoundingBoxShading(paint, state);
         }
         else
         {
-            return linearGradientUserSpaceOnUseShading(paint, state);
+            shading = linearGradientUserSpaceOnUseShading(paint, state);
         }
+        return state.shadingMaskModifier.applyMasking(state, shading);
     }
 
     private PDShading linearGradientObjectBoundingBoxShading(Paint paint, PaintApplierState state)
@@ -476,8 +609,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         // will display it another.
         float calculatedX = (float) Math.min(startPoint.getX(), endPoint.getX());
         float calculatedY = (float) Math.max(1.0f, Math.max(startPoint.getY(), endPoint.getY()));
-        float calculatedWidth = Math
-                .max(1.0f, Math.abs((float) (endPoint.getX() - startPoint.getX())));
+        float calculatedWidth = Math.max(1.0f,
+                Math.abs((float) (endPoint.getX() - startPoint.getX())));
         float negativeHeight =
                 -1.0f * Math.max(1.0f, Math.abs((float) (endPoint.getY() - startPoint.getY())));
 
@@ -534,10 +667,15 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
     {
         PDShadingType3 shading = new PDShadingType3(new COSDictionary());
         Color[] colors = getPropertyValue(paint, "getColors");
-        Color firstColor = colors[0];
-		PDColor firstColorMapped = state.colorMapper.mapColor(firstColor, state.colorMapperEnv);
-        applyAsStrokingColor(firstColor, state);
         float[] fractions = getPropertyValue(paint, "getFractions");
+        PDColor firstColorMapped = mapFirstColorOfGradient(state, colors);
+
+        if (haveColorsTransparency(colors))
+        {
+            PdfBoxGraphics2DColor[] alphaGrayscaleColors = mapAlphaToGrayscale(colors);
+            state.shadingMaskModifier = new CreateAlphaShadingMask(fractions, alphaGrayscaleColors);
+        }
+
         PDFunctionType3 type3 = buildType3Function(colors, fractions, state);
         shading.setAntiAlias(true);
         shading.setShadingType(PDShading.SHADING_TYPE2);
@@ -547,6 +685,15 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         return shading;
     }
 
+    private PDColor mapFirstColorOfGradient(PaintApplierState state, Color[] colors)
+            throws IOException
+    {
+        Color firstColor = colors[0];
+        PDColor firstColorMapped = state.colorMapper.mapColor(firstColor, state.colorMapperEnv);
+        applyAsStrokingColor(firstColor, state);
+        return firstColorMapped;
+    }
+
     private COSArray setupExtends()
     {
         COSArray extend = new COSArray();
@@ -640,9 +787,11 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
          * Reflection to access the fields of both these classes.
          */
         Color[] colors = getPropertyValue(paint, "getColors");
-        Color firstColor = colors[0];
-		PDColor firstColorMapped = state.colorMapper.mapColor(firstColor, state.colorMapperEnv);
-        applyAsStrokingColor(firstColor, state);
+        PDColor firstColorMapped = mapFirstColorOfGradient(state, colors);
+        float[] fractions = getPropertyValue(paint, "getFractions");
+        Point2D centerPoint = clonePoint((Point2D) getPropertyValue(paint, "getCenterPoint"));
+        Point2D focusPoint = clonePoint((Point2D) getPropertyValue(paint, "getFocusPoint"));
+        float radius = getPropertyValue(paint, "getRadius");
 
         /*
          * When doing a shading paint, we need to always walk the shape first.
@@ -653,24 +802,20 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         shading.setAntiAlias(true);
         shading.setShadingType(PDShading.SHADING_TYPE3);
         shading.setColorSpace(firstColorMapped.getColorSpace());
-        float[] fractions = getPropertyValue(paint, "getFractions");
-        Point2D centerPoint = clonePoint((Point2D) getPropertyValue(paint, "getCenterPoint"));
-        Point2D focusPoint = clonePoint((Point2D) getPropertyValue(paint, "getFocusPoint"));
         AffineTransform gradientTransform = getPropertyValue(paint, "getTransform");
         state.tf.concatenate(gradientTransform);
         state.tf.transform(centerPoint, centerPoint);
         state.tf.transform(focusPoint, focusPoint);
 
-        float radius = getPropertyValue(paint, "getRadius");
         radius = (float) Math.abs(radius * state.tf.getScaleX());
 
         COSArray coords = new COSArray();
 
-        coords.add(new COSFloat((float) centerPoint.getX()));
-        coords.add(new COSFloat((float) centerPoint.getY()));
-        coords.add(new COSFloat(0));
         coords.add(new COSFloat((float) focusPoint.getX()));
         coords.add(new COSFloat((float) focusPoint.getY()));
+        coords.add(new COSFloat(0));
+        coords.add(new COSFloat((float) centerPoint.getX()));
+        coords.add(new COSFloat((float) centerPoint.getY()));
         coords.add(new COSFloat(radius));
         shading.setCoords(coords);
 
@@ -678,16 +823,46 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
 
         shading.setFunction(type3);
         shading.setExtend(setupExtends());
-        return shading;
+
+        if (haveColorsTransparency(colors))
+        {
+            PdfBoxGraphics2DColor[] alphaGrayscaleColors = mapAlphaToGrayscale(colors);
+            state.shadingMaskModifier = new CreateAlphaShadingMask(fractions, alphaGrayscaleColors);
+        }
+
+        return state.shadingMaskModifier.applyMasking(state, shading);
+    }
+
+    static PdfBoxGraphics2DColor mapAlphaToGrayscale(Color c)
+    {
+        return new PdfBoxGraphics2DColor(
+                new PDColor(new float[] { (c.getAlpha() / 255f) }, PDDeviceGray.INSTANCE));
+    }
+
+    static PdfBoxGraphics2DColor[] mapAlphaToGrayscale(Color[] colors)
+    {
+        PdfBoxGraphics2DColor[] ret = new PdfBoxGraphics2DColor[colors.length];
+        for (int i = 0; i < ret.length; i++)
+        {
+            ret[i] = mapAlphaToGrayscale(colors[i]);
+        }
+        return ret;
     }
 
     private PDShading buildGradientShading(GradientPaint gradientPaint, PaintApplierState state)
             throws IOException
     {
+        Point2D startPoint = gradientPaint.getPoint1();
+        Point2D endPoint = gradientPaint.getPoint2();
+
         Color[] colors = new Color[] { gradientPaint.getColor1(), gradientPaint.getColor2() };
-        Color firstColor = colors[0];
-		PDColor firstColorMapped = state.colorMapper.mapColor(firstColor, state.colorMapperEnv);
-        applyAsStrokingColor(firstColor, state);
+        PDColor firstColorMapped = mapFirstColorOfGradient(state, colors);
+
+        if (haveColorsTransparency(colors))
+        {
+            PdfBoxGraphics2DColor[] alphaGrayscaleColors = mapAlphaToGrayscale(colors);
+            state.shadingMaskModifier = new CreateAlphaShadingMask(null, alphaGrayscaleColors);
+        }
 
         /*
          * When doing a shading paint, we need to always walk the shape first.
@@ -700,9 +875,6 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         float[] fractions = new float[] { 0, 1 };
         PDFunctionType3 type3 = buildType3Function(colors, fractions, state);
 
-        Point2D startPoint = gradientPaint.getPoint1();
-        Point2D endPoint = gradientPaint.getPoint2();
-
         state.tf.transform(startPoint, startPoint);
         state.tf.transform(endPoint, endPoint);
 
@@ -710,7 +882,7 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
 
         shading.setFunction(type3);
         shading.setExtend(setupExtends());
-        return shading;
+        return state.shadingMaskModifier.applyMasking(state, shading);
     }
 
     private void applyTexturePaint(TexturePaint texturePaint, PaintApplierState state)
@@ -738,8 +910,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         PDPageContentStream imageContentStream = new PDPageContentStream(state.document, appearance,
                 ((COSStream) pattern.getCOSObject()).createOutputStream());
         BufferedImage texturePaintImage = texturePaint.getImage();
-        PDImageXObject imageXObject = state.imageEncoder
-                .encodeImage(state.document, imageContentStream, texturePaintImage);
+        PDImageXObject imageXObject = state.imageEncoder.encodeImage(state.document,
+                imageContentStream, texturePaintImage, state.imageEncoderEnv);
 
         float ratioW = (float) ((anchorRect.getWidth()) / texturePaintImage.getWidth());
         float ratioH = (float) ((anchorRect.getHeight()) / texturePaintImage.getHeight());
@@ -787,7 +959,7 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
 
         List<Color> colorList = new ArrayList<Color>(Arrays.asList(colors));
         COSArray bounds = new COSArray();
-        if (Math.abs(fractions[0]) > EPSILON)
+        if (needBoundsKeyFrameEntry(fractions))
         {
             /*
              * We need to insert a "keyframe" for fraction 0. See also java.awt.LinearGradientPaint for future information
@@ -824,6 +996,13 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         return type3;
     }
 
+    private boolean needBoundsKeyFrameEntry(float[] fractions)
+    {
+        if (fractions == null)
+            return false;
+        return Math.abs(fractions[0]) > EPSILON;
+    }
+
     /**
      * Build a type2 function to interpolate between the given colors.
      *
@@ -842,8 +1021,8 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         for (int i = 1; i < colors.size(); i++)
         {
             Color color = colors.get(i);
-			PDColor prevPdColor = state.colorMapper.mapColor(prevColor, state.colorMapperEnv);
-			PDColor pdColor = state.colorMapper.mapColor(color, state.colorMapperEnv);
+            PDColor prevPdColor = state.colorMapper.mapColor(prevColor, state.colorMapperEnv);
+            PDColor pdColor = state.colorMapper.mapColor(color, state.colorMapperEnv);
             COSArray c0 = new COSArray();
             COSArray c1 = new COSArray();
             for (float component : prevPdColor.getComponents())
@@ -1003,4 +1182,168 @@ public class PdfBoxGraphics2DPaintApplier implements IPdfBoxGraphics2DPaintAppli
         }
     }
 
+    private final class CreateAlphaShadingMask implements ShadingMaskModifier
+    {
+        private final float[] fractions;
+        private final PdfBoxGraphics2DColor[] alphaGrayscaleColors;
+
+        public CreateAlphaShadingMask(float[] fractions,
+                PdfBoxGraphics2DColor[] alphaGrayscaleColors)
+        {
+            this.fractions = fractions;
+            this.alphaGrayscaleColors = alphaGrayscaleColors;
+        }
+
+        private final static boolean USE_PATTERN = false;
+
+        @Override
+        public PDShading applyMasking(PaintApplierState state, PDShading shading) throws IOException
+        {
+            PDShading translatedShading = createMaskShading(state, shading);
+
+            PDRectangle bbox = state.env.getGraphics2D().bbox;
+            PDAppearanceStream appearance = new PDAppearanceStream(state.document);
+            PDFormXObject xFormObject;
+            xFormObject = appearance;
+            xFormObject.setResources(new PDResources());
+            xFormObject.setBBox(bbox);
+            xFormObject.setFormType(1);
+
+            PDShadingPattern pattern = new PDShadingPattern();
+            pattern.setShading(translatedShading);
+            COSName tilingPatternName = xFormObject.getResources().add(pattern);
+            PDPattern patternCS = new PDPattern(null);
+            PDColor patternColor = new PDColor(tilingPatternName, patternCS);
+
+            PDPageContentStream contentStream = new PDPageContentStream(state.document, appearance,
+                    xFormObject.getStream().createOutputStream(COSName.FLATE_DECODE));
+            contentStream.saveGraphicsState();
+            PDExtendedGraphicsState gfxState = new PDExtendedGraphicsState();
+            gfxState.setNonStrokingAlphaConstant(1f);
+            gfxState.setStrokingAlphaConstant(1f);
+            contentStream.setGraphicsStateParameters(gfxState);
+            //contentStream.transform(new Matrix(state.tf));
+            contentStream.addRect(0, 0, bbox.getWidth(), bbox.getHeight());
+            if (!USE_PATTERN)
+            {
+                contentStream.shadingFill(translatedShading);
+            }
+            else
+            {
+                contentStream.setNonStrokingColor(patternColor);
+                contentStream.fill();
+            }
+            contentStream.restoreGraphicsState();
+            contentStream.close();
+
+            /*
+             * And now apply it as mask
+             */
+            COSDictionary group = new COSDictionary();
+            group.setItem(COSName.S, COSName.TRANSPARENCY);
+            group.setItem(COSName.CS, COSName.DEVICEGRAY);
+            group.setItem(COSName.TYPE, COSName.GROUP);
+            xFormObject.getCOSObject().setItem(COSName.GROUP, group);
+            state.resources.add(xFormObject);
+
+            state.ensureExtendedState();
+            state.pdExtendedGraphicsState.setAlphaSourceFlag(false);
+            state.pdExtendedGraphicsState.setNonStrokingAlphaConstant(null);
+            state.pdExtendedGraphicsState.setStrokingAlphaConstant(null);
+
+            COSDictionary mask = new COSDictionary();
+            mask.setItem(COSName.G, xFormObject);
+            mask.setItem(COSName.S, COSName.LUMINOSITY);
+            mask.setItem(COSName.TYPE, COSName.MASK);
+
+            state.dictExtendedState.setItem(COSName.SMASK, mask);
+
+            return shading;
+        }
+
+        private PDShading createMaskShading(PaintApplierState state, PDShading shading)
+                throws IOException
+        {
+            PDFCloneUtility pdfCloneUtility = new PDFCloneUtility(state.document);
+            COSDictionary shadingDictionary = (COSDictionary) pdfCloneUtility.cloneForNewDocument(
+                    shading.getCOSObject());
+            COSArray functions = (COSArray) shadingDictionary.getItem(COSName.FUNCTIONS);
+            if (functions != null)
+            {
+                int colorIdx = 0;
+                for (int i = 0; i < functions.size(); i++)
+                {
+                    colorIdx = patchFunction(colorIdx, (COSDictionary) functions.get(i));
+                }
+            }
+            else
+            {
+                COSDictionary function = (COSDictionary) shadingDictionary.getItem(
+                        COSName.FUNCTION);
+                patchFunction(0, function);
+            }
+            PDShading translatedShading = PDShading.create(shadingDictionary);
+            translatedShading.setColorSpace(PDDeviceGray.INSTANCE);
+            return translatedShading;
+        }
+
+        private int patchFunction(int colorIdx, COSDictionary cosBase)
+        {
+            int functionType = cosBase.getInt(COSName.FUNCTION_TYPE);
+            switch (functionType)
+            {
+            case 3:
+                /*
+                 * Combined Function
+                 */
+                COSArray functions = (COSArray) cosBase.getItem(COSName.FUNCTIONS);
+                for (int i = 0; i < functions.size(); i++)
+                {
+                    colorIdx = patchFunction(colorIdx, (COSDictionary) functions.get(i));
+                }
+                break;
+            case 2:
+                /*
+                 * Linear interpolation
+                 */
+                final float alpha0;
+                final float alpha1;
+                if (needBoundsKeyFrameEntry(fractions))
+                {
+                    if (0 == colorIdx)
+                    {
+                        alpha0 = 1f;
+                    }
+                    else
+                    {
+                        PdfBoxGraphics2DColor clr = alphaGrayscaleColors[colorIdx - 1];
+                        alpha0 = clr.toPDColor().getComponents()[0];
+                    }
+
+                    PdfBoxGraphics2DColor clr = alphaGrayscaleColors[colorIdx];
+                    alpha1 = clr.toPDColor().getComponents()[0];
+                    colorIdx++;
+                }
+                else
+                {
+
+                    PdfBoxGraphics2DColor clr = alphaGrayscaleColors[colorIdx];
+                    alpha0 = clr.toPDColor().getComponents()[0];
+                    if (colorIdx + 1 < alphaGrayscaleColors.length)
+                        clr = alphaGrayscaleColors[colorIdx + 1];
+                    alpha1 = clr.toPDColor().getComponents()[0];
+                    colorIdx++;
+                }
+
+                COSArray c0Array = new COSArray();
+                COSArray c1Array = new COSArray();
+                c0Array.add(new COSFloat(alpha0));
+                c1Array.add(new COSFloat(alpha1));
+                cosBase.setItem(COSName.C0, c0Array);
+                cosBase.setItem(COSName.C1, c1Array);
+                break;
+            }
+            return colorIdx;
+        }
+    }
 }
