@@ -94,6 +94,7 @@ public class PdfBoxGraphics2dTest extends PdfBoxGraphics2DTestBase
                 gfx.fill(new Rectangle.Float(10, 10, 100, 50));
                 gfx.fill(new Rectangle.Float(120, 10, 50, 50));
                 gfx.fill(new Rectangle.Float(200, 10, 50, 100));
+
                 RadialGradientPaint radialGradientPaint = new RadialGradientPaint(200, 200, 200,
                         new float[] { 0.0f, .2f, .4f, .9f, 1f },
                         new Color[] { Color.YELLOW, Color.GREEN, Color.RED, Color.BLUE,
@@ -106,6 +107,75 @@ public class PdfBoxGraphics2dTest extends PdfBoxGraphics2DTestBase
         });
     }
 
+    @Test
+    public void testTransparentGradients()
+    {
+        exportGraphic("simple", "transparentgradients", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx)
+            {
+                gfx.setPaint(new GradientPaint(0, 0, new Color(128, 128, 255, 128), 300, 250,
+                        new Color(0xFF, 11, 11, 5)));
+                gfx.fillRect(0, 0, 300, 250);
+
+                LinearGradientPaint linearGradientPaint = new LinearGradientPaint(0, 0, 100, 200,
+                        new float[] { 0.0f, .2f, 1f },
+                        new Color[] { new Color(255, 255, 255), new Color(255, 255, 136, 128),
+                                new Color(85, 255, 255, 0) });
+                gfx.setPaint(linearGradientPaint);
+                gfx.fill(new Rectangle.Float(10, 10, 100, 50));
+                gfx.fill(new Rectangle.Float(120, 10, 50, 50));
+                gfx.fill(new Rectangle.Float(200, 10, 50, 100));
+
+                gfx.setColor(Color.BLACK);
+                gfx.draw(new Rectangle.Float(10, 10, 100, 50));
+                gfx.draw(new Rectangle.Float(120, 10, 50, 50));
+                gfx.draw(new Rectangle.Float(200, 10, 50, 100));
+
+                RadialGradientPaint radialGradientPaint = new RadialGradientPaint(200, 200, 200,
+                        new float[] { 0.0f, .2f, 1f },
+                        new Color[] { new Color(255, 255, 255), new Color(255, 255, 136, 128),
+                                new Color(85, 255, 255, 0) });
+
+                gfx.setPaint(radialGradientPaint);
+                gfx.fill(new Rectangle.Float(10, 120, 100, 50));
+                gfx.fill(new Rectangle.Float(120, 120, 50, 50));
+                gfx.fill(new Rectangle.Float(200, 120, 50, 100));
+
+                gfx.setColor(Color.BLACK);
+                gfx.draw(new Rectangle.Float(10, 120, 100, 50));
+                gfx.draw(new Rectangle.Float(120, 120, 50, 50));
+                gfx.draw(new Rectangle.Float(200, 120, 50, 100));
+            }
+        });
+        exportGraphic("simple", "transparentgradients_inverse", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx)
+            {
+                gfx.setPaint(new GradientPaint(300, 250, new Color(128, 128, 255, 128), 0, 0,
+                        new Color(0xFF, 11, 11, 5)));
+                gfx.fillRect(0, 0, 300, 250);
+
+                LinearGradientPaint linearGradientPaint = new LinearGradientPaint(100, 200, 0, 0,
+                        new float[] { 0.0f, .2f, 1f },
+                        new Color[] { new Color(255, 255, 255), new Color(255, 255, 136, 128),
+                                new Color(85, 255, 255, 0) });
+                gfx.setPaint(linearGradientPaint);
+                gfx.fill(new Rectangle.Float(10, 10, 100, 50));
+                gfx.fill(new Rectangle.Float(120, 10, 50, 50));
+                gfx.fill(new Rectangle.Float(200, 10, 50, 100));
+
+                gfx.setColor(Color.BLACK);
+                gfx.draw(new Rectangle.Float(10, 10, 100, 50));
+                gfx.draw(new Rectangle.Float(120, 10, 50, 50));
+                gfx.draw(new Rectangle.Float(200, 10, 50, 100));
+
+            }
+        });
+    }
+
     @Test
     public void testBuildPatternFill()
     {
@@ -233,6 +303,29 @@ public class PdfBoxGraphics2dTest extends PdfBoxGraphics2DTestBase
         });
     }
 
+    @Test
+    public void testImageInterpolation()
+    {
+        exportGraphic("imageenc", "imageinterpolation", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx) throws IOException
+            {
+                BufferedImage img2 = ImageIO.read(
+                        PdfBoxGraphics2dTest.class.getResourceAsStream("pixeltest.png"));
+                gfx.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+                        RenderingHints.VALUE_INTERPOLATION_BICUBIC);
+                gfx.drawImage(img2, 30, 10, 75, 50, null);
+                gfx.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+                        RenderingHints.VALUE_INTERPOLATION_BILINEAR);
+                gfx.drawImage(img2, 30, 90, 75, 50, null);
+                gfx.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+                        RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
+                gfx.drawImage(img2, 30, 160, 75, 50, null);
+            }
+        });
+    }
+
     @Test
     public void testEvenOddRules()
     {
@@ -392,24 +485,104 @@ public class PdfBoxGraphics2dTest extends PdfBoxGraphics2DTestBase
         });
     }
 
-	@Test
-	public void testLineWithRotation() {
-		exportGraphic("simple", "lineWithRotation", new GraphicsExporter() {
-			@Override
-			public void draw(Graphics2D gfx) {
-				gfx.setStroke(new BasicStroke(5f));
+    @Test
+    public void testLineWithRotation()
+    {
+        exportGraphic("simple", "lineWithRotation", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx)
+            {
+                gfx.setStroke(new BasicStroke(5f));
 
-				float centerX = 200;
-				float centerY = 200;
+                float centerX = 200;
+                float centerY = 200;
 
-				gfx.translate(centerX, centerY);
-				for (int i = 0; i < 360; i += 10) {
-					gfx.rotate(10 / 360f * Math.PI * 2);
-					gfx.setColor(debugColors[(i / 10) % debugColors.length]);
-					gfx.draw(new Line2D.Double(0, 0, 50, 0));
-				}
-			}
-		});
-	}
+                gfx.translate(centerX, centerY);
+                for (int i = 0; i < 360; i += 10)
+                {
+                    gfx.rotate(10 / 360f * Math.PI * 2);
+                    gfx.setColor(debugColors[(i / 10) % debugColors.length]);
+                    gfx.draw(new Line2D.Double(0, 0, 50, 0));
+                }
+            }
+        });
+    }
 
+    @Test
+    public void testRadialFocialPointTransparency()
+    {
+        exportGraphic("simple", "radialFocialPointTransparency", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx)
+            {
+                drawFocialRadialWithTransparency(gfx, 100, 100, 100);
+                drawFocialRadialWithTransparency(gfx, 200, 200, 20);
+                drawFocialRadialWithTransparency(gfx, 250, 200, 150);
+            }
+        });
+    }
+
+    @Test
+    public void testRadialFocialPointSolid()
+    {
+        exportGraphic("simple", "radialFocialPointSolid", new GraphicsExporter()
+        {
+            @Override
+            public void draw(Graphics2D gfx)
+            {
+                drawFocialRadialSolid(gfx, 100, 100, 100);
+                drawFocialRadialSolid(gfx, 200, 200, 20);
+                drawFocialRadialSolid(gfx, 250, 200, 150);
+            }
+        });
+    }
+
+    private static void drawFocialRadialWithTransparency(Graphics2D g2d, float x, float y,
+            int diameter)
+    {
+        Composite cRestore = g2d.getComposite();
+        AffineTransform tRestore = g2d.getTransform();
+        g2d.translate(x, y);
+        g2d.setColor(Color.RED);
+        g2d.fillOval(0, 0, diameter, diameter);
+
+        // Setup radial gradient
+        Point2D center = new Point2D.Float(diameter / 2f, diameter / 2f);
+        float radius = diameter / 2f;
+        Point2D focus = new Point2D.Float((int) (diameter * .3), (int) (diameter * .3));
+        float[] dist = { 0.0f, .75f, 1f };
+        Color c = Color.WHITE;
+        Color transparent = new Color(c.getRed(), c.getGreen(), c.getBlue(), 0);
+        Color[] colors = { c, transparent, Color.BLACK };
+        RadialGradientPaint p = new RadialGradientPaint(center, radius, focus, dist, colors,
+                MultipleGradientPaint.CycleMethod.NO_CYCLE);
+        g2d.setPaint(p);
+        g2d.fillOval(0, 0, diameter, diameter);
+
+        g2d.setComposite(cRestore);
+        g2d.setTransform(tRestore);
+    }
+
+    private static void drawFocialRadialSolid(Graphics2D g2d, float x, float y, int diameter)
+    {
+        Composite cRestore = g2d.getComposite();
+        AffineTransform tRestore = g2d.getTransform();
+        g2d.translate(x, y);
+
+        // Setup radial gradient
+        Point2D center = new Point2D.Float(diameter / 2f, diameter / 2f);
+        float radius = diameter / 2f;
+        Point2D focus = new Point2D.Float((int) (diameter * .3), (int) (diameter * .3));
+        float[] dist = { 0.0f, .75f, 1f };
+        Color[] colors = { Color.RED, Color.WHITE, Color.BLACK };
+        RadialGradientPaint p = new RadialGradientPaint(center, radius, focus, dist, colors,
+                MultipleGradientPaint.CycleMethod.NO_CYCLE);
+        g2d.setPaint(p);
+        g2d.fillOval(0, 0, diameter, diameter);
+
+        g2d.setComposite(cRestore);
+        g2d.setTransform(tRestore);
+    }
 }
